#!/usr/bin/perl -w 
#
# bwConfig.pm
#
# Support for config files
#
# Copyright (c) 2000-2002 William E. Weinman  http://bw.org/    
# All rights reserved. 
#

package bwConfig;
use strict;
use IO::File;

our $VERSION = 0.1;

# constants
my $TRUE = 1;
my $FALSE = '';

sub new
{
my $proto = shift;
my $class = ref($proto) || $proto;
my $self = {};
bless ($self, $class);
$self->init(@_);

return $self;
}

sub init
{
my $self = shift;
return $TRUE if $self->{INIT};  # only allow one init
$self->{INIT} = $TRUE;

# arrange the room
$self->{RAW_LINES} = [];
$self->{KEYWORDS} = [];
$self->{VALUES} = {};
$self->{ARRAYS} = {};

# shall we parse? 
if(@_) {
  $self->parse(shift);
  }

return $TRUE;
}

sub filename
{
my $self = shift;
$self->{FILENAME} = shift if @_;
return $self->{FILENAME};
}

sub version
{
return $VERSION;
}

sub error
{
my $self = shift;
if(@_) {
  $self->{ERROR} = shift;
  }
return $self->{ERROR};
}

sub keywords
{
my $self = shift;
return $self->{KEYWORDS} || {};
}

sub values
{
my $self = shift;
return $self->{VALUES} || {};
}

sub arrays
{
my $self = shift;
return $self->{ARRAYS} || {};
}

sub parse
{
my $self = shift;
$self->error('bw::Config: No filename.') and return $FALSE unless @_;
my $fh = new IO::File($self->filename(shift), 'r') 
  or $self->error("Cannot open config file ($self->{FILENAME}): $!");
return $FALSE unless $fh;

while(<$fh>) {

  s/#.*//;       # trim end-of-line comments
  s/^\s+//;      # trim leading ...
  s/\s+$//;      # ... and trailing whitespace
  next if /^$/;  # skip semantically empty lines

  push @{$self->{RAW_LINES}}, $_;
  my ($lh, $rh) = split(/\s+/, $_, 2);
  push @{$self->{KEYWORDS}}, $lh;
  $rh = '' unless $rh;             # avoid undef
  $self->{VALUES}{$lh} = $rh;
  @{$self->{ARRAYS}{$lh}} = split(/:/, $rh);
  }
return $TRUE;
}


1;

__END__

=pod

=head1 NAME

bw::Config.pm

This document describes bw::Config version 0.1.

=head1 SYNOPSIS

  use bw::Config;
  my $config = new bw::Config( filename );

  my $config = new bw::Config;
  $config->parse( filename );

=head1 DESCRIPTION

bw::Config reads and parses simple configuration files. 

=head1 METHODS

=over 4

=item version

Return the version string.

=item filename

Set or return the filename used by bw::Config->parse.

=item keywords

Return an arrayref of keywords, in the order they appear in the 
config file. 

=item values

Return a hashref of key/value pairs. 

=item arrays

Return a hashref of key/arrayref pairs for multi-item values.

=item error

Return the latest error condition.

=back

=head1 FILE FORMAT

The config file format is very simple. It's a line-oriented format 
where keys and values are separated by whitespace. 

  key value

Array values (multiple values per key) are separated by colons. 

  key value:value:value:value

Lines that begin with '#' are ignored.

  # this is a comment 

anything after '#' to the end of the line is considered a comment and
is ignored.

  key value # comment comment blah blah blah

All leading and trailing whitespace are trimed, and a key without 
a value is assumed to have a zero-length string as the value.

=head1 NOTES

=item *

Values are never undef. A zero-length string is provided for null content. 

=item *

Errors are reported via the bw::Config->error method. It's a good idea to 
check this before using the values from your config file. 

  if(my $errorstring = $config->error) {
    die $errorstring;  # or some other way of reporting the error
    }

=head1 BUGS AND MISFEATURES

There is no quoting mechanism for the config file. The colon-separated 
arrays are particularly weak. There should be a way to quote a colon 
so it's not interpreted as such. 

There should also be a way to quote strings and quote marks. I will add 
these features in a future version. 

=head1 AUTHOR

Bill Weinman (http://bw.org/)

=cut

