#!/usr/bin/perl -w 
#
# bwEmail.pm
#
# Support for email messages
#
# Copyright (c) 2000-2002 William E. Weinman  http://bw.org/    
# All rights reserved. 
#

package bwEmail;
use strict;
our $VERSION = 0.1;

# constants
my $CRLF = "\x0d\x0a";
my $TRUE = 1;
my $FALSE = '';

sub new
{
my $proto = shift;
my $class = ref($proto) || $proto;
my $self = shift || {};

bless($self, $class);

$self->init or return '';

return $self;
}

sub init
{
my $self = shift;

$self->{SMTP_HOST} = $::SMTP_HOST || "127.0.0.1" ;
$self->{SMTP_PORT} = 25;
$self->{TIMEOUT} = 10;   # 10 seconds: web users tend to be impatient

$self->{RECEIVED_FROM} = $ENV{REMOTE_ADDR} || '';
$self->{RECEIVED_FROM} .= ' (' . $ENV{REMOTE_HOST} . ')' if $ENV{REMOTE_HOST};
$self->{RECEIVED_FROM} .= ' (UID ' . $::user->id . ')' if $::user;
$self->{RECEIVED_WITH} = $ENV{SERVER_PROTOCOL} ? 
  "$ENV{SERVER_PROTOCOL} ($ENV{GATEWAY_INTERFACE}/$ENV{REQUEST_METHOD})" : '';
$self->{RECEIVED_OKAY} = $TRUE if $self->{RECEIVED_FROM};
$self->{HELO} = $ENV{HTTP_HOST} || $ENV{SERVER_NAME} || "mail";
$self->{SMTP_DATE} = $self->smtpdate;
$self->{EXTRA_HEADERS} = {};

$self->{RC} = [];

return $TRUE;
}

sub version
{
return $VERSION;
}

sub error
{
my $self = shift;
if(@_) {
  $self->{ERROR} = shift;
  }
return $self->{ERROR};
}

sub validate_email
{
my $email = shift;

if(ref($email)) {  # allow for object or direct
  $email = shift;
  }

return $FALSE unless $email;

# this should really do a DNS test too.
if($email =~ /^[^\x00-\x20()\<\>\[\]\@\,\;\:\\\/"]+\@[^\x00-\x20()\<\>\[\]\@\,\;\:\\\/"]+$/i) { return $TRUE; }
else { return $FALSE }
}

sub smtphost
{
my $self = shift;
$self->{SMTP_HOST} = shift if @_;
return $self->{SMTP_HOST};
}

# smtpdate
#   returns a formatted date string suitable for SMTP
#
sub smtpdate
{
my $self = shift;
my $t = shift || time;
my @days = qw( Sun Mon Tue Wed Thu Fri Sat );
my @months = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );
my $i;
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
my @gm = gmtime($t);
my $hoffset = sprintf("%+2.02d00", ($i = ($hour - $gm[2])) > 12 ? ($i - 24) : $i );
return sprintf("%s, %d %s %d %02d:%02d:%02d $hoffset",
  $days[$wday], $mday, $months[$mon], $year + 1900, $hour, $min, $sec);
}

sub header
{
my $self = shift;

if(@_) {
  my $header_name = shift;
  my $header_value = shift || '';
  $self->{EXTRA_HEADERS}->{$header_name} = $header_value;
  }
return $self->{EXTRA_HEADERS};
}

sub headers { header(@_) }

sub from
{
my $self = shift;
my $from = shift if @_;
$self->{FROM} = $from if $from;
return $self->{FROM};
}

sub from_name
{
my $self = shift;
my $name = shift if @_;
$self->{FROM_NAME} = $name if $name;
return $self->{FROM_NAME};
}

sub from_line
{
my $self = shift;
$self->{FROM_LINE} = $self->from_name ? $self->from_name . " <" . $self->from . ">" : $self->from; 
return $self->{FROM_LINE};
}

sub subject
{
my $self = shift;
my $subj = shift if @_;
$self->{SUBJECT} = $subj if $subj;
return $self->{SUBJECT};
}

sub date
{
my $self = shift;
return $self->{SMTP_DATE};
}

sub body
{
my $self = shift;
my $body = shift if @_;
$self->{BODY} = $body if $body;
return $self->{BODY};
}

sub to
{
my $self = shift;
my $to = shift if @_;
$self->{TO} = $to if $to;
return $self->{TO};
}

sub to_name
{
my $self = shift;
my $name = shift if @_;
$self->{TO_NAME} = $name if $name;
return $self->{TO_NAME};
}

sub to_line
{
my $self = shift;
$self->{TO_LINE} = $self->to_name ? $self->to_name . " <" . $self->to . ">" : $self->to; 
return $self->{TO_LINE};
}

sub return_path
{
my $self = shift;
my $rp = shift;
$self->{RETURN_PATH} = $rp if $rp;
return $self->{RETURN_PATH} || $self->{FROM} || '';
}

sub message
{
my $self = shift;
my $s = '';

my $body = $self->{BODY};

# this ensures that there are no bare linefeeds anywhere in the message. 
# it seems a bit extreme, but it's the only way I could find that worked. 
if($body) {
  my @body = split (/\x0a/, $body);
  grep { s/\x0d$// } @body;  # loose any extraneous CRs
  $body = join($CRLF, @body);
  }

return $self->error("cannot build message without both FROM and TO") unless ($self->{FROM} and $self->{TO});

my $extra_headers = $self->headers;
my @top_headers = qw( Return-Path Errors-To );

foreach my $h ( @top_headers ) {
  $s .= "${h}: " . $extra_headers->{$h} . $CRLF if $extra_headers->{$h};
  }
$s .= 'Received: ' . $self->received . $CRLF if $self->{RECEIVED_OKAY};

foreach my $h ( keys %$extra_headers ) {
  next if grep { $h eq $_ } @top_headers;
  $s .= $h . ": " . $extra_headers->{$h} . $CRLF;
  }

$s .= 'Date: ' . $self->date . $CRLF;
$s .= 'Subject: ' . $self->{SUBJECT} . $CRLF if $self->{SUBJECT};
$s .= 'From: ' . $self->from_line . $CRLF;
$s .= 'To: ' . $self->to_line . $CRLF;
$s .= $CRLF;
$s .= $body . $CRLF if $body;
return $s;
}

sub received
{
my $self = shift;
my $s = '';

$s .= "from " . $self->{RECEIVED_FROM} if $self->{RECEIVED_FROM};
$s .= $CRLF . "  " if $s and $self->{HELO};
$s .= "by " . $self->{HELO} if $self->{HELO};
$s .= $CRLF . "  " if $s && $self->{RECEIVED_WITH};
$s .= "with " . $self->{RECEIVED_WITH} if $self->{RECEIVED_WITH};
$s .= ";$CRLF  " if $s;
$s .= $self->{SMTP_DATE};

return $s;
}

sub rc_line
{
my $self = shift;
my $line = shift || '';
my $socket = $self->{SOCKET};

$self->{SMTP_RESULT} = 0;
$self->{SMTP_RESULT_TEXT} = '';
$self->{SMTP_RESULT_LINE} = '';

while($line =~ /\d{3}-(.*)/) {
  $self->{SMTP_RESULT_TEXT} .= $1;
  $line = $socket->getline;
  }

$line =~ s/[\x0d\x0a]+$//;
push @{$self->{RC}}, $line;

my ($lh,$rh) = split(/ /, $line, 2);
$self->{SMTP_RESULT} .= $lh || 0;
$self->{SMTP_RESULT_TEXT} .= $rh || '';
$self->{SMTP_RESULT_LINE} .= $line;

return $self->{RC};
}

sub rc
{
my $self = shift;
return $self->{RC};
}

sub make_smtp_socket
{
my $self = shift;
use IO::Socket::INET;

my $s = new IO::Socket::INET( 
  PeerAddr => $self->{SMTP_HOST}, 
  PeerPort => $self->{SMTP_PORT},
  Proto => 'tcp',
  Timeout => $self->{TIMEOUT}
  );

$self->error("make_smtp_socket: $!") unless $s;
return undef if $self->error;

# autoflush is already set in later versions of the IO library, but we do 
# it here anyway -- it's cheap insurance
$s->autoflush(1);

$self->{SOCKET} = $s;
}

sub smtp_lineout
{
my $self = shift;
my $line = shift;
my $socket = $self->{SOCKET};
$socket->print($line . $CRLF);
$self->rc_line($socket->getline);
}

sub smtp_transaction
{
my $self = shift;
my $socket = $self->{SOCKET};
my $rc = $self->{RC};

$self->rc_line(scalar <$socket>);   # get the SMTP signon
$self->error(qq{SMTP Connect: SMTP server said "$self->{SMTP_RESULT_LINE}", quitting.})
  unless $self->{SMTP_RESULT} == 220;
return $rc if $self->error;

# HELO
$self->smtp_lineout("HELO $self->{HELO}");
$self->error(qq{SMTP HELO: SMTP server said "$self->{SMTP_RESULT_LINE}", quitting.})
  unless $self->{SMTP_RESULT} == 250;
return $rc if $self->error;

# MAIL FROM
$self->smtp_lineout("MAIL FROM:<" . $self->return_path . ">");
$self->error(qq{SMTP MAIL FROM: SMTP server said "$self->{SMTP_RESULT_LINE}", quitting.})
  unless ($self->{SMTP_RESULT} >= 250 and $self->{SMTP_RESULT} < 260);
return $rc if $self->error;

# RCPT TO
$self->smtp_lineout("RCPT TO:<$self->{TO}>");
$self->error(qq{SMTP RCPT: SMTP server said "$self->{SMTP_RESULT_LINE}", quitting.})
  unless ($self->{SMTP_RESULT} >= 250 and $self->{SMTP_RESULT} < 260);
return $rc if $self->error;

# Send the DATA command
$self->smtp_lineout('DATA');
$self->error(qq{SMTP DATA: SMTP server said "$self->{SMTP_RESULT_LINE}", quitting.})
  unless $self->{SMTP_RESULT} == 354;
return $rc if $self->error;

# send the message itself
$socket->print( $self->message . "${CRLF}.${CRLF}" );
$self->rc_line($socket->getline);
$self->error(qq{SMTP DATA End: SMTP server said "$self->{SMTP_RESULT_LINE}", quitting.})
  unless $self->{SMTP_RESULT} == 250;
return $rc if $self->error;

# Done: send QUIT
# no need to check the value of the return code. 
$self->smtp_lineout('QUIT');

$socket->close;

return $rc;
}

sub send
{
my $self = shift;
my $message = $self->message;

if($self->make_smtp_socket) {
  $self->smtp_transaction;
  }
}

1;

__END__

=head1 bw::Email.pm

Email-related library. 

=head1 new

Crate a new bw::Email object. 

=head1 init

Initializations called by new().

=head1 version

Return the version string.

=head1 error

Return the latest error condition.

=head1 smtphost

Set and/or return the SMTP host. 

=head1 smtpdate

Return a date formatted for SMTP.

=head1 header ( name, value )

Create a new header. 

=head1 headers

Alias for header().

=head1 from

Set and/or return the From: email address. 

=head1 from_name

Set and/or return the From: name. 

=head1 from_line

Return the From: header value. 

=head1 subject

Set and/or return the Subject. 

=head1 date

Returns the SMTP date. 

=head1 body

Set and/or return the body of the email message. 

=head1 to

Set and/or return the To: email address. 

=head1 to_name

Set and/or return the To: name. 

=head1 to_line

Return the value of the To: header. 

=head1 return_path

Set and/or return the return-path (envelope address). 

=head1 message

Return the fully-assembled email message. 

=head1 received

Assemble the Received: header. (Internal only).

=head1 rc_line

Used internally for the SMTP transaction. 

=head1 rc

Used internally for the SMTP transaction. 

=head1 make_smtp_socket

Used internally for the SMTP transaction. 

=head1 smtp_lineout

Used internally for the SMTP transaction. 

=head1 smtp_transaction

Used internally for the SMTP transaction. 

=head1 send

Send the message. 

=cut

